@@ -5,6 +5,8 @@ import yaml
 from patchwork.common.utils.progress_bar import PatchflowProgressBar
 from patchwork.step import Step
 from patchwork.steps import (
+    LLM,
+    PR,
     CallLLM,
     CommitChanges,
     CreateIssueComment,
@@ -59,7 +61,7 @@ class ResolveIssue(Step):
         outputs = ReadIssues(self.inputs).run()
         self.inputs.update(outputs)
 
-        self.inputs["texts"] = self.inputs["issue_text"]
+        self.inputs["texts"] = self.inputs.get("issue_body") or self.inputs.get("issue_title")
 
         outputs = QueryEmbeddings(self.inputs).run()
         self.inputs.update(outputs)
@@ -93,7 +95,7 @@ The following files in the repository may be relevant to the issue:
                     "startLine": 0,
                     "endLine": len(lines),
                     "affectedCode": file_content,
-                    "messageText": "\n".join(self.inputs["texts"]),
+                    "messageText": self.inputs["texts"],
                 }
             )
 
@@ -103,11 +105,7 @@ The following files in the repository may be relevant to the issue:
         self.inputs["response_partitions"] = {
             "patch": ["Fixed Code:", "```", "\n", "```"],
         }
-        outputs = PreparePrompt(self.inputs).run()
-        self.inputs.update(outputs)
-        outputs = CallLLM(self.inputs).run()
-        self.inputs.update(outputs)
-        outputs = ExtractModelResponse(self.inputs).run()
+        outputs = LLM(self.inputs).run()
         self.inputs.update(outputs)
 
         # Modify code files with the suggested changes
@@ -116,11 +114,7 @@ The following files in the repository may be relevant to the issue:
 
         # Commit changes and create PR
         self.inputs["pr_header"] = f'This pull request from patchwork fixes {self.inputs["issue_url"]}.'
-        outputs = CommitChanges(self.inputs).run()
-        self.inputs.update(outputs)
-        outputs = PreparePR(self.inputs).run()
-        self.inputs.update(outputs)
-        outputs = CreatePR(self.inputs).run()
+        outputs = PR(self.inputs).run()
         self.inputs.update(outputs)
 
         return self.inputs
