@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import json
 import os
+from itertools import islice
 from pathlib import Path
 from pprint import pformat
 from textwrap import indent
@@ -18,8 +19,8 @@ _DEFAULT_PATCH_URL = "https://patchwork.patched.codes/v1"
 
 
 class LLMModel(Protocol):
-    def call(self, prompts) -> list[str]:
-        pass
+    def call(self, prompts: list[dict]) -> list[str]:
+        ...
 
 
 class CallGemini(LLMModel):
@@ -41,7 +42,7 @@ class CallGemini(LLMModel):
         self.api_key = key
         self.allow_truncated = allow_truncated
 
-    def call(self, prompts):
+    def call(self, prompts: list[dict]):
         contents = []
         for prompt in prompts:
             texts = [dict(text=subprompt.get("content", "")) for subprompt in prompt]
@@ -94,12 +95,16 @@ class CallOpenAI(LLMModel):
         self.allow_truncated = allow_truncated
         self.client = OpenAI(api_key=key, **client_args)
 
-    def call(self, prompts) -> list[str]:
+    def call(self, prompts: list[dict]) -> list[str]:
         contents = []
+
+        # Parse model arguments
+        parsed_model_args = self.parse_model_args(self.model_args)
+
         for prompt in prompts:
             logger.debug(f"Message sent: \n{indent(pformat(prompt), '  ')}")
             try:
-                completion = self.client.chat.completions.create(model=self.model, messages=prompt, **self.model_args)
+                completion = self.client.chat.completions.create(model=self.model, messages=prompt, **parsed_model_args)
             except Exception as e:
                 logger.error(e)
                 completion = None
@@ -124,6 +129,31 @@ class CallOpenAI(LLMModel):
 
         return contents
 
+    def parse_model_args(self, model_args: dict) -> dict:
+        # List of arguments in their respective types
+        int_args = {"max_tokens", "n", "top_logprobs"}
+        float_args = {"temperature", "top_p", "presence_penalty", "frequency_penalty"}
+        bool_args = {"logprobs"}
+
+        new_model_args = dict()
+        for key, arg in model_args.items():
+            if key in int_args and isinstance(arg, str):
+                try:
+                    new_model_args[key] = int(arg)
+                except ValueError:
+                    logger.warning(f"Failed to parse {key} as integer. Removing from arguments.")
+            elif key in float_args and isinstance(arg, str):
+                try:
+                    new_model_args[key] = float(arg)
+                except ValueError:
+                    logger.warning(f"Failed to parse {key} as float. Removing from arguments.")
+            elif key in bool_args and isinstance(arg, str):
+                new_model_args[key] = arg.lower() == "true"
+            else:
+                new_model_args[key] = arg
+
+        return new_model_args
+
 
 class CallLLM(Step):
     def __init__(self, inputs: dict):
@@ -147,8 +177,10 @@ class CallLLM(Step):
         else:
             raise ValueError('Missing required data: "prompt_file" or "prompts"')
 
+        self.call_limit = int(inputs.get("max_llm_calls", 50))
         self.model_args = {key[len("model_") :]: value for key, value in inputs.items() if key.startswith("model_")}
         self.client_args = {key[len("client_") :]: value for key, value in inputs.items() if key.startswith("client_")}
+        self.save_responses_to_file = inputs.get("save_responses_to_file", None)
 
         llm_key = inputs.get("openai_api_key") or os.environ.get("OPENAI_API_KEY")
 
@@ -189,7 +221,23 @@ class CallLLM(Step):
         )
 
     def run(self) -> dict:
-        contents = self.llm.call(self.prompts)
+        contents = self.llm.call(list(islice(self.prompts, self.call_limit)))
+
+        if self.save_responses_to_file:
+            # Convert relative path to absolute path
+            file_path = os.path.abspath(self.save_responses_to_file)
+
+            mode = "a" if os.path.exists(file_path) else "w"
+
+            with open(file_path, mode) as f:
+                for prompt, response in zip(self.prompts, contents):
+                    data = {
+                        "model": self.llm.model,
+                        "model_args": self.llm.model_args,
+                        "request": prompt,
+                        "response": response,
+                    }
+                    f.write(json.dumps(data) + "\n")
 
         logger.info(f"Run completed {self.__class__.__name__}")
-        return dict(new_code=contents, openai_responses=contents)
+        return dict(openai_responses=contents)
